@@ -2,7 +2,7 @@
 
 #include "../util/util_time.h"
 
-#include "../util/sync/sync_signal_win32.h"
+#include "../util/sync/sync_signal.h"
 
 #include "d3d11_context.h"
 #include "d3d11_state_object.h"
@@ -118,8 +118,8 @@ namespace dxvk {
     DxvkCsThread m_csThread;
     bool         m_csIsBusy = false;
 
-    Rc<sync::Win32Fence> m_eventSignal;
-    uint64_t             m_eventCount = 0;
+    Rc<sync::CallbackFence> m_eventSignal;
+    uint64_t                m_eventCount = 0;
 
     dxvk::high_resolution_clock::time_point m_lastFlush
       = dxvk::high_resolution_clock::now();
