@@ -58,6 +58,20 @@ namespace dxvk {
       return read(data);
     }
 
+    bool read(DxvkIlBinding& data, uint32_t version) {
+      if (version < 10) {
+        DxvkIlBindingV9 v9;
+
+        if (!read(v9))
+          return false;
+
+        data = v9.convert();
+        return true;
+      }
+
+      return read(data);
+    }
+
     template<typename T>
     bool write(const T& data) {
       if (m_size + sizeof(T) > MaxSize)
@@ -196,8 +210,8 @@ namespace dxvk {
   
 
   DxvkStateCache::~DxvkStateCache() {
-    { std::lock_guard<std::mutex> workerLock(m_workerLock);
-      std::lock_guard<std::mutex> writerLock(m_writerLock);
+    { std::lock_guard<dxvk::mutex> workerLock(m_workerLock);
+      std::lock_guard<dxvk::mutex> writerLock(m_writerLock);
 
       m_stopThreads.store(true);
 
@@ -230,7 +244,7 @@ namespace dxvk {
     }
 
     // Queue a job to write this pipeline to the cache
-    std::unique_lock<std::mutex> lock(m_writerLock);
+    std::unique_lock<dxvk::mutex> lock(m_writerLock);
 
     m_writerQueue.push({ shaders, state,
       DxvkComputePipelineStateInfo(),
@@ -254,7 +268,7 @@ namespace dxvk {
     }
 
     // Queue a job to write this pipeline to the cache
-    std::unique_lock<std::mutex> lock(m_writerLock);
+    std::unique_lock<dxvk::mutex> lock(m_writerLock);
 
     m_writerQueue.push({ shaders,
       DxvkGraphicsPipelineStateInfo(), state,
@@ -270,11 +284,11 @@ namespace dxvk {
       return;
     
     // Add the shader so we can look it up by its key
-    std::unique_lock<std::mutex> entryLock(m_entryLock);
+    std::unique_lock<dxvk::mutex> entryLock(m_entryLock);
     m_shaderMap.insert({ key, shader });
 
     // Deferred lock, don't stall workers unless we have to
-    std::unique_lock<std::mutex> workerLock;
+    std::unique_lock<dxvk::mutex> workerLock;
 
     auto pipelines = m_pipelineMap.equal_range(key);
 
@@ -290,7 +304,7 @@ namespace dxvk {
         continue;
       
       if (!workerLock)
-        workerLock = std::unique_lock<std::mutex>(m_workerLock);
+        workerLock = std::unique_lock<dxvk::mutex>(m_workerLock);
       
       m_workerQueue.push(item);
     }
@@ -849,7 +863,8 @@ namespace dxvk {
         in.gpState.rsCullMode,
         in.gpState.rsFrontFace,
         in.gpState.rsViewportCount,
-        in.gpState.rsSampleCount);
+        in.gpState.rsSampleCount,
+        VK_CONSERVATIVE_RASTERIZATION_MODE_DISABLED_EXT);
 
       out.gpState.ms = DxvkMsInfo(
         in.gpState.msSampleCount,
@@ -906,7 +921,7 @@ namespace dxvk {
     while (!m_stopThreads.load()) {
       WorkerItem item;
 
-      { std::unique_lock<std::mutex> lock(m_workerLock);
+      { std::unique_lock<dxvk::mutex> lock(m_workerLock);
 
         if (m_workerQueue.empty()) {
           m_workerBusy -= 1;
@@ -939,7 +954,7 @@ namespace dxvk {
     while (!m_stopThreads.load()) {
       DxvkStateCacheEntry entry;
 
-      { std::unique_lock<std::mutex> lock(m_writerLock);
+      { std::unique_lock<dxvk::mutex> lock(m_writerLock);
 
         m_writerCond.wait(lock, [this] () {
           return m_writerQueue.size()
@@ -970,12 +985,7 @@ namespace dxvk {
     if (!path.empty() && *path.rbegin() != '/')
       path += '/';
     
-    std::string exeName = env::getExeName();
-    auto extp = exeName.find_last_of('.');
-    
-    if (extp != std::string::npos && exeName.substr(extp + 1) == "exe")
-      exeName.erase(extp);
-    
+    std::string exeName = env::getExeBaseName();
     path += exeName + ".dxvk-cache";
     return str::tows(path.c_str());
   }
