@@ -1,7 +1,9 @@
+#include <array>
 #include <fstream>
 #include <sstream>
 #include <iostream>
 #include <regex>
+#include <utility>
 
 #include "config.h"
 
@@ -145,9 +147,12 @@ namespace dxvk {
     { R"(\\Raft\.exe$)", {{
       { "dxvk.enableOpenVR",                "False" },
     }} },
-    /* Crysis 3 - slow if it notices AMD card     */
+    /* Crysis 3 - slower if it notices AMD card     *
+     * Apitrace mode helps massively in cpu bound   *
+     * game parts                                   */
     { R"(\\Crysis3\.exe$)", {{
       { "dxgi.customVendorId",              "10de" },
+      { "d3d11.apitraceMode",               "True" },
     }} },
     /* Atelier series - games try to render video *
      * with a D3D9 swap chain over the DXGI swap  *
@@ -209,6 +214,10 @@ namespace dxvk {
     { R"(\\KingdomCome\.exe$)", {{
       { "d3d11.apitraceMode",               "True" },
     }} },
+    /* Homefront: The Revolution                  */
+    { R"(\\Homefront2_Release\.exe$)", {{
+      { "d3d11.apitraceMode",               "True" },
+    }} },
     /* Sniper Ghost Warrior Contracts             */
     { R"(\\SGWContracts\.exe$)", {{
       { "d3d11.apitraceMode",               "True" },
@@ -444,6 +453,14 @@ namespace dxvk {
       { "dxgi.tearFree",                    "False" },
       { "dxgi.syncInterval",                "1"     },
     }} },
+    /* Sine Mora EX */
+    { R"(\\SineMoraEX\.exe$)", {{
+      { "d3d9.maxFrameRate",                "60" },
+    }} },
+    /* Fantasy Grounds                           */
+    { R"(\\FantasyGrounds\.exe$)", {{
+      { "d3d9.noExplicitFrontBuffer",       "True" },
+    }} },
   }};
 
 
@@ -558,15 +575,13 @@ namespace dxvk {
   bool Config::parseOptionValue(
     const std::string&  value,
           bool&         result) {
-    if (value == "True") {
-      result = true;
-      return true;
-    } else if (value == "False") {
-      result = false;
-      return true;
-    } else {
-      return false;
-    }
+    static const std::array<std::pair<const char*, bool>, 2> s_lookup = {{
+      { "true",  true  },
+      { "false", false },
+    }};
+
+    return parseStringOption(value,
+      s_lookup.begin(), s_lookup.end(), result);
   }
 
 
@@ -605,18 +620,34 @@ namespace dxvk {
   bool Config::parseOptionValue(
     const std::string&  value,
           Tristate&     result) {
-    if (value == "True") {
-      result = Tristate::True;
-      return true;
-    } else if (value == "False") {
-      result = Tristate::False;
-      return true;
-    } else if (value == "Auto") {
-      result = Tristate::Auto;
-      return true;
-    } else {
-      return false;
+    static const std::array<std::pair<const char*, Tristate>, 3> s_lookup = {{
+      { "true",  Tristate::True  },
+      { "false", Tristate::False },
+      { "auto",  Tristate::Auto  },
+    }};
+
+    return parseStringOption(value,
+      s_lookup.begin(), s_lookup.end(), result);
+  }
+
+
+  template<typename I, typename V>
+  bool Config::parseStringOption(
+          std::string   str,
+          I             begin,
+          I             end,
+          V&            value) {
+    std::transform(str.begin(), str.end(), str.begin(),
+      [] (unsigned char c) { return (c >= 'A' && c <= 'Z') ? (c + 'a' - 'A') : c; });
+
+    for (auto i = begin; i != end; i++) {
+      if (str == i->first) {
+        value = i->second;
+        return true;
+      }
     }
+
+    return false;
   }
 
 
